/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package com.mycompany.prj3.prjwebservicerestxmljson;

import com.mycompany.exception.ExceptionUtil;
import com.mycompany.prj3.beans.Project;
import java.net.URI;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.ws.rs.Consumes;
import javax.ws.rs.DELETE;
import javax.ws.rs.GET;
import javax.ws.rs.POST;
import javax.ws.rs.PUT;
import javax.ws.rs.Path;
import javax.ws.rs.Produces;
import javax.ws.rs.QueryParam;
import javax.ws.rs.WebApplicationException;
import javax.ws.rs.core.Context;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import javax.ws.rs.core.UriInfo;
import org.apache.log4j.Logger;

/**
 *
 * @author thierry
 */
@Stateless
@Path("project")
public class ProjectResource {

    private static final Logger logger = Logger.getLogger(ProjectResource.class);

    @Context
    private UriInfo uriInfo;

    @PersistenceContext(unitName = "puBdProject")
    private EntityManager entityManager;

    public ProjectResource() {
    }

    @GET
    @Produces(MediaType.APPLICATION_XML)
    public Response getProjectAsXml(@QueryParam("id") Long id) {
        logger.info("In getProjectAsXml for id : " + id);
        Project project = null;
        try {
            project = entityManager.find(Project.class, id);
        } catch (Exception e) {
            logger.error("Exception catchee " + ExceptionUtil.displayException(e));
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
        logger.info("Project name : " + (project != null ? project.getName() : "Pas de projet"));
        if (project != null) {
            return Response.ok(project,
                    MediaType.APPLICATION_XML).build();
        } else {
            throw new WebApplicationException(Response.Status.NOT_FOUND);
        }
    }

    @GET
    @Produces(MediaType.APPLICATION_JSON)
    public Response getProjectAsJson(@QueryParam("id") Long id) {
        logger.info("In getProjectAsJson for id : " + id);
        Project project = null;
        try {
            project = entityManager.find(Project.class, id);
        } catch (Exception e) {
            logger.error("Exception catchee " + ExceptionUtil.displayException(e));
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
        logger.info("Project name : " + (project != null ? project.getName() : "Pas de projet"));

        if (project != null) {
            return Response.ok(project,
                    MediaType.APPLICATION_JSON).build();
        } else {
            throw new WebApplicationException(Response.Status.NOT_FOUND);
        }
    }

    @GET
    @Produces("text/xml")
    @Path("/all/")
    public List<Project> getProjects() {
        logger.info("In getProjects");
        List<Project> list = null;
        try {
            list = entityManager.createNamedQuery(Project.PROJECT_FIND_ALL_PROJECTS).getResultList();
            logger.info("Nombre de projet : " + (list != null ? list.size() : "0"));
        } catch (Exception e) {
            logger.error("Exception catchee " + ExceptionUtil.displayException(e));
        }

        return list;
    }

    @POST
    @Consumes({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON})
    public Response insertProject(Project project) {
        logger.info("In insertProject");
        try {
            entityManager.persist(project);
            logger.info("projectName=[" + project.getName() + "] id=" + project.getProjectId());
            logger.info("startDate=" + (project.getStartDate() != null ? project.getStartDate() : "Pas de date"));

            URI bookUri = uriInfo.getAbsolutePathBuilder().path(project.getProjectId().toString()).build();
            return Response.created(bookUri).build();
        } catch (Exception e) {
            logger.error("Exception catchee " + ExceptionUtil.displayException(e));
            throw new WebApplicationException(Response.Status.INTERNAL_SERVER_ERROR);
        }
    }

    @PUT
    @Consumes({MediaType.APPLICATION_XML, MediaType.APPLICATION_JSON})
    public void updateProject(Project project) {
        logger.info("In updateProject");
        try {
            entityManager.merge(project);
            logger.info("projectName=[" + project.getName() + "] id=" + project.getProjectId());
        } catch (Exception e) {
            logger.error("Exception catchee " + ExceptionUtil.displayException(e));
        }
    }

    @DELETE
    @Consumes("text/xml")
    public void deleteProject(@QueryParam("id") Long id) {
        logger.info("In deleteProject for id " + id);
        try {
            Project project = entityManager.find(Project.class, id);
            logger.info("projectName=[" + project.getName() + "] id=" + project.getProjectId());
            entityManager.remove(project);
        } catch (Exception e) {
            logger.error("Exception catchee " + ExceptionUtil.displayException(e));
        }
    }
}
